#include<stdio.h>
#include<R.h>
#include<Rmath.h>
#include<stdlib.h>

#define MINF -1.0e15

// log bivariate density (see Padoan et al 2010 eqn. 4). 
//Smith model
//a=( h^TSigma^{-1} h )^1/2 with ||.|| the euclidean norm
void smith_logdens_and_dervs(double *z, 
															double *aIN, 
															double *a1IN, double *a2IN, double *a3IN, 
															double *a11IN, double *a22IN, double *a33IN, 
															double *a12IN, double *a13IN, double *a23IN, 
															double *s1, double *s2, double *s12, 
															double *llik, double *score, double *hessian, 
															int *br, int j, int *n)
{
	//the log bivariate desnity is of the form
	//-A-B+log(C*D+E)
	//A, B, C, D, E depend of the entries of S, on z, and on h
	if(!ISNA(z[0]) && !ISNA(z[1]))
	{
		double w, v, a=*aIN, ai=1.0/a;
		double z0i=1.0/z[0], z0sqi=1.0/(z[0]*z[0]), 
				   z1i=1.0/z[1], z1sqi=1.0/(z[1]*z[1]), 
				   z0z1i=1.0/(z[0]*z[1]);
		double log_zjzi=log(z[1]/z[0]);

		w = 0.5*a + log_zjzi*ai;
		v = a-w;

			double A, A1, A2, A3,
					   B, B1, B2, B3,
					   C, C1, C2, C3,
					   D, D1, D2, D3,
					   E, E1, E2, E3;
			double F1w, F2w, F3w, 
					   F1v, F2v, F3v; 
			double G1w, G2w, G3w, 
					   G1v, G2v, G3v; 
			double w1, w2, w3,
					   v1, v2, v3,
					   a1=*a1IN, a2=*a2IN, a3=*a3IN;

			double asq=a*a;
			double asqi=1.0/asq, 
						 athrd=asq*a, 
						 athrdi=1.0/athrd, 
						 afth=asq*asq*a,
						 asxti=1.0/(asq*asq*asq);

			double dnorm_w, dnorm_v,
		  		   der_dnorm_w, der_dnorm_v;
			double cons = 1.0/2.506628275; // 1/sqrt(2*pi)


			dnorm_w = exp(-w*w*0.5)*cons;
			dnorm_v = exp(-v*v*0.5)*cons;	
	
			A = pnorm5(w, 0.0, 1.0, 1, 0)*z0i;
			B = pnorm5(v, 0.0, 1.0, 1, 0)*z1i;
			C = A*z0i + dnorm_w*ai*z0sqi - dnorm_v*ai*z0z1i;
			D = B*z1i + dnorm_v*ai*z1sqi - dnorm_w*ai*z0z1i;
			E = (v*dnorm_w*z0i + w*dnorm_v*z1i)*z0z1i*asqi;

			//contribution to the pairwise loglikelihood by j-th pair
			*llik+=-A-B+log(C*D+E);

			if(*br)
			{
			///Compute score+hessian

			///SCORE
			//let theta=(s1,s2,s12)
			//the derivative of the log bivariate density wrt the t-th component of theta is
			//-At-Bt+(D*Ct+C*Dt+Et)/(C*D+E)
			//t=1,2,3

			w1 = a1*0.5 - log_zjzi*a1*asqi;
			w2 = a2*0.5 - log_zjzi*a2*asqi;
			w3 = a3*0.5 - log_zjzi*a3*asqi;		

			v1 = a1-w1;
			v2 = a2-w2;
			v3 = a3-w3;
			
			der_dnorm_w = -dnorm_w*w;
			der_dnorm_v = -dnorm_v*v;
	
			A1 = 	dnorm_w*w1*z0i;
			A2 = 	dnorm_w*w2*z0i;
			A3 = 	dnorm_w*w3*z0i;

			B1 = 	dnorm_v*v1*z1i;
			B2 = 	dnorm_v*v2*z1i;
			B3 = 	dnorm_v*v3*z1i;			

			F1w = (der_dnorm_w*w1*a-dnorm_w*a1)*asqi;
			F2w = (der_dnorm_w*w2*a-dnorm_w*a2)*asqi;
			F3w = (der_dnorm_w*w3*a-dnorm_w*a3)*asqi;

			F1v = (der_dnorm_v*v1*a-dnorm_v*a1)*asqi;
			F2v = (der_dnorm_v*v2*a-dnorm_v*a2)*asqi;
			F3v = (der_dnorm_v*v3*a-dnorm_v*a3)*asqi;

			C1 = A1*z0i + F1w*z0sqi - F1v*z0z1i;
			C2 = A2*z0i + F2w*z0sqi - F2v*z0z1i;
			C3 = A3*z0i + F3w*z0sqi - F3v*z0z1i;

			D1 = B1*z1i + F1v*z1sqi - F1w*z0z1i;
			D2 = B2*z1i + F2v*z1sqi - F2w*z0z1i;
			D3 = B3*z1i + F3v*z1sqi - F3w*z0z1i;			

			G1w = ((v1*dnorm_w + v*der_dnorm_w*w1)*a - 2.0*v*dnorm_w*a1)*athrdi;
			G2w = ((v2*dnorm_w + v*der_dnorm_w*w2)*a - 2.0*v*dnorm_w*a2)*athrdi;
			G3w = ((v3*dnorm_w + v*der_dnorm_w*w3)*a - 2.0*v*dnorm_w*a3)*athrdi;			

			G1v = ((w1*dnorm_v + w*der_dnorm_v*v1)*a - 2.0*w*dnorm_v*a1)*athrdi;
			G2v = ((w2*dnorm_v + w*der_dnorm_v*v2)*a - 2.0*w*dnorm_v*a2)*athrdi;
			G3v = ((w3*dnorm_v + w*der_dnorm_v*v3)*a - 2.0*w*dnorm_v*a3)*athrdi;

			E1 = G1w*z0sqi*z1i + G1v*z1sqi*z0i;
			E2 = G2w*z0sqi*z1i + G2v*z1sqi*z0i;
			E3 = G3w*z0sqi*z1i + G3v*z1sqi*z0i;

			//score function
			//s1
			score[j] += -A1-B1 + (D*C1+C*D1+E1)/(C*D+E);
			//s2
			score[j+*n] += -A2-B2 + (D*C2+C*D2+E2)/(C*D+E);	
			//s12
			score[j+2**n] += -A3-B3 + (D*C3+C*D3+E3)/(C*D+E);


			//HESSIAN
			//let theta=(s1,s2,s12)
			//the second-order partial derivatives of the log bivariate density wrt the tk-th components of theta are
			//-(D*Ct+C*Dt+Et)(D*Ck+C*Dk+Ek)/(C*D+E)^2 - Atk - Btk + (Ct*Dk+Ck*Dt+Ctk*D+C*Dtk+Etk)/(C*D+E)
			//t,k=1,2,3

			double w11, w22, w33, v11, v22, v33, a11=*a11IN, a22=*a22IN, a33=*a33IN;
			double w12, w13, w23, v12, v13, v23, a12=*a12IN, a13=*a13IN, a23=*a23IN;
			double A11, A22, A33, B11, B22, B33, C11, C22, C33, D11, D22, D33, E11, E22, E33;
			double A12, A13, A23, B12, B13, B23, C12, C13, C23, D12, D13, D23, E12, E13, E23;
			double F11w, F22w, F33w, F11v, F22v, F33v;
			double F12w, F13w, F23w, F12v, F13v, F23v;
			double G11w, G22w, G33w, G11v, G22v, G33v; 	 
			double G12w, G13w, G23w, G12v, G13v, G23v; 	 
			double der2_dnorm_w, der2_dnorm_v;
			double a1sq=a1*a1, a2sq=a2*a2, a3sq=a3*a3;
			double w1sq, w2sq, w3sq,
					   v1sq, v2sq, v3sq;

			der2_dnorm_w = -dnorm_w*(1.0-w*w);
			der2_dnorm_v = -dnorm_v*(1.0-v*v);

			w11 = a11*0.5 - log_zjzi*(a11*a-2.0*a1sq)*athrdi;
			w22 = a22*0.5 - log_zjzi*(a22*a-2.0*a2sq)*athrdi;
			w33 = a33*0.5 - log_zjzi*(a33*a-2.0*a3sq)*athrdi;			
			w12 = a12*0.5 - log_zjzi*(a12*a-2.0*a1*a2)*athrdi;
			w13 = a13*0.5 - log_zjzi*(a13*a-2.0*a1*a3)*athrdi;
			w23 = a23*0.5 - log_zjzi*(a23*a-2.0*a2*a3)*athrdi;

			w1sq=w1*w1;
			w2sq=w2*w2;
			w3sq=w3*w3;			


			v11 = a11-w11;
			v22 = a22-w22;
			v33 = a33-w33;
			v12 = a12-w12;
			v13 = a13-w13;
			v23 = a23-w23;						
				
			v1sq=v1*v1;
			v2sq=v2*v2;
			v3sq=v3*v3;
						

			A11 = (der_dnorm_w*w1sq+dnorm_w*w11)*z0i;	
			A22 = (der_dnorm_w*w2sq+dnorm_w*w22)*z0i;	
			A33 = (der_dnorm_w*w3sq+dnorm_w*w33)*z0i;	

			A12 = (der_dnorm_w*w1*w2+dnorm_w*w12)*z0i;	
			A13 = (der_dnorm_w*w1*w3+dnorm_w*w13)*z0i;	
			A23 = (der_dnorm_w*w2*w3+dnorm_w*w23)*z0i;	


			B11 = (der_dnorm_v*v1sq+dnorm_v*v11)*z1i;	
			B22 = (der_dnorm_v*v2sq+dnorm_v*v22)*z1i;	
			B33 = (der_dnorm_v*v3sq+dnorm_v*v33)*z1i;	
						
			B12 = (der_dnorm_v*v1*v2+dnorm_v*v12)*z1i;	
			B13 = (der_dnorm_v*v1*v3+dnorm_v*v13)*z1i;	
			B23 = (der_dnorm_v*v2*v3+dnorm_v*v23)*z1i;				


			F11w = (der2_dnorm_w*w1sq*a+der_dnorm_w*w11*a+der_dnorm_w*w1*a1-der_dnorm_w*w1*a1-dnorm_w*a11-2.0*a*a1*F1w)*asqi;
			F22w = (der2_dnorm_w*w2sq*a+der_dnorm_w*w22*a+der_dnorm_w*w2*a2-der_dnorm_w*w2*a2-dnorm_w*a22-2.0*a*a2*F2w)*asqi;
			F33w = (der2_dnorm_w*w3sq*a+der_dnorm_w*w33*a+der_dnorm_w*w3*a3-der_dnorm_w*w3*a3-dnorm_w*a33-2.0*a*a3*F3w)*asqi;			

			F12w = (der2_dnorm_w*w1*w2*a+der_dnorm_w*w12*a+der_dnorm_w*w1*a2-der_dnorm_w*w2*a1-dnorm_w*a12-2.0*a*a2*F1w)*asqi;
			F13w = (der2_dnorm_w*w1*w3*a+der_dnorm_w*w13*a+der_dnorm_w*w1*a3-der_dnorm_w*w3*a1-dnorm_w*a13-2.0*a*a3*F1w)*asqi;
			F23w = (der2_dnorm_w*w2*w3*a+der_dnorm_w*w23*a+der_dnorm_w*w2*a3-der_dnorm_w*w3*a2-dnorm_w*a23-2.0*a*a3*F2w)*asqi;

	
			F11v = (der2_dnorm_v*v1sq*a+der_dnorm_v*v11*a+der_dnorm_v*v1*a1-der_dnorm_v*v1*a1-dnorm_v*a11-2.0*a*a1*F1v)*asqi;
			F22v = (der2_dnorm_v*v2sq*a+der_dnorm_v*v22*a+der_dnorm_v*v2*a2-der_dnorm_v*v2*a2-dnorm_v*a22-2.0*a*a2*F2v)*asqi;
			F33v = (der2_dnorm_v*v3sq*a+der_dnorm_v*v33*a+der_dnorm_v*v3*a3-der_dnorm_v*v3*a3-dnorm_v*a33-2.0*a*a3*F3v)*asqi;
						
			F12v = (der2_dnorm_v*v1*v2*a+der_dnorm_v*v12*a+der_dnorm_v*v1*a2-der_dnorm_v*v2*a1-dnorm_v*a12-2.0*a*a2*F1v)*asqi;
			F13v = (der2_dnorm_v*v1*v3*a+der_dnorm_v*v13*a+der_dnorm_v*v1*a3-der_dnorm_v*v3*a1-dnorm_v*a13-2.0*a*a3*F1v)*asqi;
			F23v = (der2_dnorm_v*v2*v3*a+der_dnorm_v*v23*a+der_dnorm_v*v2*a3-der_dnorm_v*v3*a2-dnorm_v*a23-2.0*a*a3*F2v)*asqi;			
			

			C11 = A11*z0i + F11w*z0sqi - F11v*z0z1i;
			C22 = A22*z0i + F22w*z0sqi - F22v*z0z1i;
			C33 = A33*z0i + F33w*z0sqi - F33v*z0z1i;

			C12 = A12*z0i + F12w*z0sqi - F12v*z0z1i;
			C13 = A13*z0i + F13w*z0sqi - F13v*z0z1i;
			C23 = A23*z0i + F23w*z0sqi - F23v*z0z1i;			


			D11 = B11*z1i + F11v*z1sqi - F11w*z0z1i;
			D22 = B22*z1i + F22v*z1sqi - F22w*z0z1i;
			D33 = B33*z1i + F33v*z1sqi - F33w*z0z1i;

			D12 = B12*z1i + F12v*z1sqi - F12w*z0z1i;
			D13 = B13*z1i + F13v*z1sqi - F13w*z0z1i;
			D23 = B23*z1i + F23v*z1sqi - F23w*z0z1i;


		G11w = ( ( (v11*dnorm_w + v1*der_dnorm_w*w1 + v1*der_dnorm_w*w1 + v*der2_dnorm_w*w1sq + v*der_dnorm_w*w11)*a +  (v1*dnorm_w+v*der_dnorm_w*w1)*a1 - 2.0*v1*dnorm_w*a1 - 2.0*v*dnorm_w*a11 - 2.0*v*der_dnorm_w*a1*w1 )*athrd - 3.0*afth*a1*G1w )*asxti;
		G22w = ( ( (v22*dnorm_w + v2*der_dnorm_w*w2 + v2*der_dnorm_w*w2 + v*der2_dnorm_w*w2sq + v*der_dnorm_w*w22)*a +  (v2*dnorm_w+v*der_dnorm_w*w2)*a2 - 2.0*v2*dnorm_w*a2 - 2.0*v*dnorm_w*a22 - 2.0*v*der_dnorm_w*a2*w2 )*athrd - 3.0*afth*a2*G2w )*asxti;
		G33w = ( ( (v33*dnorm_w + v3*der_dnorm_w*w3 + v3*der_dnorm_w*w3 + v*der2_dnorm_w*w3sq + v*der_dnorm_w*w33)*a +  (v3*dnorm_w+v*der_dnorm_w*w3)*a3 - 2.0*v3*dnorm_w*a3 - 2.0*v*dnorm_w*a33 - 2.0*v*der_dnorm_w*a3*w3 )*athrd - 3.0*afth*a3*G3w )*asxti;

		G12w = ( ( (v12*dnorm_w + v1*der_dnorm_w*w2 + v2*der_dnorm_w*w1 + v*der2_dnorm_w*w1*w2 + v*der_dnorm_w*w12)*a +  (v1*dnorm_w+v*der_dnorm_w*w1)*a2 - 2.0*v2*dnorm_w*a1 - 2.0*v*dnorm_w*a12 - 2.0*v*der_dnorm_w*a1*w2 )*athrd - 3.0*afth*a2*G1w )*asxti;
		G13w = ( ( (v13*dnorm_w + v1*der_dnorm_w*w3 + v3*der_dnorm_w*w1 + v*der2_dnorm_w*w1*w3 + v*der_dnorm_w*w13)*a +  (v1*dnorm_w+v*der_dnorm_w*w1)*a3 - 2.0*v3*dnorm_w*a1 - 2.0*v*dnorm_w*a13 - 2.0*v*der_dnorm_w*a1*w3 )*athrd - 3.0*afth*a3*G1w )*asxti;
		G23w = ( ( (v23*dnorm_w + v2*der_dnorm_w*w3 + v3*der_dnorm_w*w2 + v*der2_dnorm_w*w2*w3 + v*der_dnorm_w*w23)*a +  (v2*dnorm_w+v*der_dnorm_w*w2)*a3 - 2.0*v3*dnorm_w*a2 - 2.0*v*dnorm_w*a23 - 2.0*v*der_dnorm_w*a2*w3 )*athrd - 3.0*afth*a3*G2w )*asxti;


		G11v = ( ( (w11*dnorm_v + w1*der_dnorm_v*v1 + w1*der_dnorm_v*v1 + w*der2_dnorm_v*v1sq + w*der_dnorm_v*v11)*a +  (w1*dnorm_v+w*der_dnorm_v*v1)*a1 - 2.0*w1*dnorm_v*a1 - 2.0*w*dnorm_v*a11 - 2.0*w*der_dnorm_v*a1*v1 )*athrd - 3.0*afth*a1*G1v )*asxti;
		G22v = ( ( (w22*dnorm_v + w2*der_dnorm_v*v2 + w2*der_dnorm_v*v2 + w*der2_dnorm_v*v2sq + w*der_dnorm_v*v22)*a +  (w2*dnorm_v+w*der_dnorm_v*v2)*a2 - 2.0*w2*dnorm_v*a2 - 2.0*w*dnorm_v*a22 - 2.0*w*der_dnorm_v*a2*v2 )*athrd - 3.0*afth*a2*G2v )*asxti;
		G33v = ( ( (w33*dnorm_v + w3*der_dnorm_v*v3 + w3*der_dnorm_v*v3 + w*der2_dnorm_v*v3sq + w*der_dnorm_v*v33)*a +  (w3*dnorm_v+w*der_dnorm_v*v3)*a3 - 2.0*w3*dnorm_v*a3 - 2.0*w*dnorm_v*a33 - 2.0*w*der_dnorm_v*a3*v3 )*athrd - 3.0*afth*a3*G3v )*asxti;

		G12v = ( ( (w12*dnorm_v + w1*der_dnorm_v*v2 + w2*der_dnorm_v*v1 + w*der2_dnorm_v*v1*v2 + w*der_dnorm_v*v12)*a +  (w1*dnorm_v+w*der_dnorm_v*v1)*a2 - 2.0*w2*dnorm_v*a1 - 2.0*w*dnorm_v*a12 - 2.0*w*der_dnorm_v*a1*v2 )*athrd - 3.0*afth*a2*G1v )*asxti;
		G13v = ( ( (w13*dnorm_v + w1*der_dnorm_v*v3 + w3*der_dnorm_v*v1 + w*der2_dnorm_v*v1*v3 + w*der_dnorm_v*v13)*a +  (w1*dnorm_v+w*der_dnorm_v*v1)*a3 - 2.0*w3*dnorm_v*a1 - 2.0*w*dnorm_v*a13 - 2.0*w*der_dnorm_v*a1*v3 )*athrd - 3.0*afth*a3*G1v )*asxti;
		G23v = ( ( (w23*dnorm_v + w2*der_dnorm_v*v3 + w3*der_dnorm_v*v2 + w*der2_dnorm_v*v2*v3 + w*der_dnorm_v*v23)*a +  (w2*dnorm_v+w*der_dnorm_v*v2)*a3 - 2.0*w3*dnorm_v*a2 - 2.0*w*dnorm_v*a23 - 2.0*w*der_dnorm_v*a2*v3 )*athrd - 3.0*afth*a3*G2v )*asxti;




		E11 = G11w*z0sqi*z1i + G11v*z1sqi*z0i;
		E22 = G22w*z0sqi*z1i + G22v*z1sqi*z0i;
		E33 = G33w*z0sqi*z1i + G33v*z1sqi*z0i;

		E12 = G12w*z0sqi*z1i + G12v*z1sqi*z0i;
		E13 = G13w*z0sqi*z1i + G13v*z1sqi*z0i;
		E23 = G23w*z0sqi*z1i + G23v*z1sqi*z0i;		
	

		//entries of the hessian
		//s1 s1
		hessian[0] += -(D*C1+C*D1+E1)*(D*C1+C*D1+E1)/((C*D+E)*(C*D+E))-A11-B11+(2.0*C1*D1+D*C11+C*D11+E11)/(C*D+E);
		//s2 s2
		hessian[1] += -(D*C2+C*D2+E2)*(D*C2+C*D2+E2)/((C*D+E)*(C*D+E))-A22-B22+(2.0*C2*D2+D*C22+C*D22+E22)/(C*D+E);
		//s3 s3
		hessian[2] += -(D*C3+C*D3+E3)*(D*C3+C*D3+E3)/((C*D+E)*(C*D+E))-A33-B33+(2.0*C3*D3+D*C33+C*D33+E33)/(C*D+E);		
		//s1 s2
		hessian[3] += -(D*C1+C*D1+E1)*(D*C2+C*D2+E2)/((C*D+E)*(C*D+E))-A12-B12+(C1*D2+C2*D1+D*C12+C*D12+E12)/(C*D+E);		
		//s1 s12
		hessian[4] += -(D*C1+C*D1+E1)*(D*C3+C*D3+E3)/((C*D+E)*(C*D+E))-A13-B13+(C1*D3+C3*D1+D*C13+C*D13+E13)/(C*D+E);		
		//s2 s12
		hessian[5] += -(D*C2+C*D2+E2)*(D*C3+C*D3+E3)/((C*D+E)*(C*D+E))-A23-B23+(C2*D3+C3*D2+D*C23+C*D23+E23)/(C*D+E);		
		}
	}
}

//pairwise-log-likelihood, score and hessian calculated on the whole sample
void smith_pwllik_dervs(double *pwlik, double *score, double *hessian, double *zIN, double *s1, double *s2, double *s12, int *sets1, int *sets2, double *t1, double *t2, int *length_sets, int *n, int *q, int *bias_reduction)
{
	int i, j;
	double z[2], h[2];
	double *a = malloc(*length_sets * sizeof(double));
	double *a1 = malloc(*length_sets * sizeof(double));
	double *a2 = malloc(*length_sets * sizeof(double));
	double *a3 = malloc(*length_sets * sizeof(double));
	double *a11 = malloc(*length_sets * sizeof(double));
	double *a22 = malloc(*length_sets * sizeof(double));
	double *a33 = malloc(*length_sets * sizeof(double));
	double *a12 = malloc(*length_sets * sizeof(double));
	double *a13 = malloc(*length_sets * sizeof(double));
	double *a23 = malloc(*length_sets * sizeof(double));
	double detSi=1.0/(*s1**s2-*s12**s12);
	double detS2i=detSi*detSi;
	double detS3i=detS2i*detSi;
	double detS4i=detS2i*detS2i;
	double detSsri=sqrt(1.0/(*s1**s2-*s12**s12));

	for(i=0; i<*length_sets; i++)
	{
		h[0] = *(t1+i);
		h[1] = *(t2+i);
		a[i] = sqrt( (h[0]*(h[0]**s2-h[1]**s12) + h[1]*(h[1]**s1-h[0]**s12) ) )*detSsri;
		a1[i] = -(h[1]**s12-h[0]**s2)*(h[1]**s12-h[0]**s2)*detS2i/(2.0*a[i]);
		a2[i] = -(-h[0]**s12+h[1]**s1)*(-h[0]**s12+h[1]**s1)*detS2i/(2.0*a[i]);
		a3[i] = (h[1]**s1-h[0]**s12)*(h[1]**s12-h[0]**s2)*detS2i/a[i];

		a11[i] = -(h[1]**s12-h[0]**s2)*(h[1]**s12-h[0]**s2)*(6.0*h[0]*h[1]**s12**s2-3.0*h[0]*h[0]**s2**s2+h[1]*h[1]*(*s12**s12-4.0**s1**s2))*detS4i/(4.0*a[i]*a[i]*a[i]);
		a22[i] = (-h[0]**s12+h[1]**s1)*(-h[0]**s12+h[1]**s1)*(-6.0*h[0]*h[1]**s12**s1+3.0*h[1]*h[1]**s1**s1-h[0]*h[0]*(*s12**s12-4.0**s1**s2))*detS4i/(4.0*a[i]*a[i]*a[i]);
		a33[i] = -( -6.0*h[0]*h[1]*h[1]*h[1]**s1**s12*(*s12**s12+*s1**s2)-6.0*h[0]*h[0]*h[0]*h[1]**s12**s2*(*s12**s12+*s1**s2) + h[1]*h[1]*h[1]*h[1]**s1**s1*(2.0**s12**s12+*s1**s2) + h[0]*h[0]*h[0]*h[0]**s2**s2*(2.0**s12**s12+*s1**s2) + h[0]*h[0]*h[1]*h[1]*(3.0**s12**s12**s12**s12+14.0**s1**s12**s12**s2+*s1**s1**s2**s2) )*detS3i/((-h[1]*h[1]**s1+2.0*h[0]*h[1]**s12-h[0]*h[0]**s2)*a[i]);
		
		a12[i] = a[i]*(h[1]**s1-h[0]**s12)*(h[1]**s12-h[0]**s2)*(h[1]**s12*(3.0*h[1]**s1-7.0*h[0]**s12)+h[0]**s2*(h[1]**s1+3.0*h[0]**s12))*detS2i/(4.0*(h[1]*h[1]**s1-2.0*h[0]*h[1]**s12+h[0]*h[0]**s2)*(h[1]*h[1]**s1-2.0*h[0]*h[1]**s12+h[0]*h[0]**s2));
		a13[i] = -(h[1]**s12-h[0]**s2)*(-3.0*h[0]*h[0]*h[0]**s12**s2**s2+h[0]*h[0]*h[1]**s2*(8.0**s12**s12+*s1**s2) + h[1]*h[1]*h[1]**s1*(*s12**s12+2.0**s1**s2)-3.0*h[0]*h[1]*h[1]**s12*(*s12**s12+2.0**s1**s2) )*detS4i/(2.0*a[i]*a[i]*a[i]);
		a23[i] = (h[0]**s12-h[1]**s1)*(3.0*h[1]*h[1]*h[1]**s12**s1**s1-h[0]*h[1]*h[1]**s1*(8.0**s12**s12+*s1**s2) - h[0]*h[0]*h[0]**s2*(*s12**s12+2.0**s1**s2)+3.0*h[0]*h[0]*h[1]**s12*(*s12**s12+2.0**s1**s2) )*detS4i/(2.0*a[i]*a[i]*a[i]);

	}

	for(j=0; j<*n; j++)
	{
		pwlik[j]=0.0;
		for(i=0; i<*length_sets; i++)
		{
			z[0] = *(zIN + *(sets1+i)-1 + j**q );
			z[1] = *(zIN + *(sets2+i)-1 + j**q );

			smith_logdens_and_dervs(z, 
															&a[i], &a1[i], &a2[i], &a3[i], 
															&a11[i], &a22[i], &a33[i], 
															&a12[i], &a13[i], &a23[i], 
															s1, s2, s12, &pwlik[j], score, hessian, bias_reduction, j, n);
		}
	}

	free(a);
	free(a1); free(a2); free(a3);
	free(a11); free(a22); free(a33);
	free(a12); free(a13); free(a23);
}




//build a regular grid
void my_grid(double *res1, double *res2, double *vec, int *len_vec)
{
	register int i, j;//, ite=0;

	for(i=0; i<*len_vec; i++)
	{
		for(j=0; j<*len_vec; j++)
		{
			*res1++ = *(vec + i);
			*res2++ = *(vec + j);
		}
	}
}


//function that defines which pairs need to be considered
void my_sets(double *loc1, double *loc2, double *d, int *lenght_loc, int *N)
{
	register int i, j;
	double t1, t2;

	char format[] = "%d";
	char filename[sizeof format+100];
	sprintf(filename,format,*N);
	FILE *file = fopen(filename,"w");	

	for(i=0; i<(*lenght_loc-1); i++)
	{
		for(j=(i+1); j<*lenght_loc; j++)
		{
			t1 = -loc1[i]+loc1[j];
			t2 = -loc2[i]+loc2[j];

			if( pow(t1*t1 + t2*t2 ,.5)<=*d)
			{
				fprintf(file,"%d %d %f %f\n",i+1, j+1, t1, t2);	
			}				
		}
	}

	fclose(file);
}

