## Copyright (C) 2022 Ioannis Kosmidis

## Distributed as part of the supplementary material for the manuscript
## "Empirical bias-reducing adjustments to estimating functions"
##
## Authors: Ioannis Kosmidis
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!
##
## It is assumed that the user has a Julia and R installation with all
## packages required by the current script and any other script
## included.
##
##
## Code used by scripts that reproduce the results in Example 5.3

using Revise, DataFrames, CSV, Graphs

include("auto_symmetric_module.jl")

# area_to_use = 3
# min_village = 26
# max_village = 36

df = CSV.read("Gambia.csv", DataFrame);
## Get the data in the central region of The Gambia (area 3)
# keep = falses(size(df)[1])
# area_inds = findall(df[:,"area"] .== area_to_use)
# keep[sample(area_inds, n_to_use, replace = false)] .= true

# keep[area_inds] .= true

keep = @. (df[:, "area"] == area_to_use) & (df[:, "village"] >= min_village) & (df[:, "village"] <= max_village)

# df = df[df[:, "area"] .== area, :];
# df = df[df[:, "village"] .>= min_village , :];
# df = df[df[:, "village"] .< 38, :];
df = df[keep, :];
df[!, "age"] = convert.(Float64, df[:, "age"]);
df[!, "age"] = df[:, "age"] / 365;

# Names of covariates to use from df
covariates = ["age", "netuse", "treated", "green", "phc"];
# Get village indices
villages = unique(df[:, "village"]);

# Construct neighbourhoods (any two children are in the same
# neighbourhood only if the come from the same village)
elist = [0 0];
for i in villages
    obs_in_i = findall(df[:, "village"] .== i)
    e = hcat(reshape([ [x, y]  for x= obs_in_i, y = obs_in_i], length(obs_in_i)^2)...)
    e = convert(Matrix, e[:, e[1, :] .!= e[2, :]]')
    global elist = vcat(elist, e)
end
elist = elist[2:size(elist)[1], :];
# elist = DataFrame(Dict("start" => elist[:, 1], "finish" => elist[:, 2]));

Adj = zeros(Int64, size(df)[1], size(df)[1])
for i in 1:size(elist)[1]
    Adj[elist[i,:1], elist[i,:2]] = 1
end


# Define G, X, and Y (the response in -1, 1 coding)
G = SimpleGraph(Adj)#MetaGraph(Gdf, :start, :finish));
X = hcat(ones(size(df)[1]), Matrix(df[:, covariates]));
Y = @. 2 * df[:, "pos"] - 1;
parnames = vcat("intercept", covariates, "lambda");

# Define the data structure
gambia_data = auto_symmetric.Data(Y, X, G);
pall = auto_symmetric.nvar(gambia_data) + 1;

# The subsets to use ifor defining emat
subsets = [findall(df[:, "village"] .== s) for s in villages];
