## Copyright (C) 2022 Ioannis Kosmidis

## Distributed as part of the supplementary material for the manuscript
## "Empirical bias-reducing adjustments to estimating functions"
##
## Authors: Ioannis Kosmidis
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!
##
## It is assumed that the user has a Julia and R installation with all
## packages required by the current script and any other script
## included.
##
##
## Code for reproducing the results in Table 5

load("images/highdim_autologistic.rda")

library("dplyr")
library("tidyr")

save_plot <- TRUE

df <- NULL
for (i in seq_along(lambdas)) {

    df_m <- data.frame(
        theta = rep(c(beta, lambdas[i]), nsamples),
        parameter = rep(1:(p + 1), nsamples),
        samples = rep(1:10, each = p + 1),
        estimate = c(coef_m[i, , ]),
        lambda = lambdas[i],
        method = "M"
    )

    df_mp <- data.frame(
        theta = rep(c(beta, lambdas[i]), nsamples),
        parameter = rep(1:(p + 1), nsamples),
        samples = rep(1:10, each = p + 1),
        estimate = c(coef_mp[i, , ]),
        lambda = lambdas[i],
        method = "Mp"
    )

    df_rbmp <- data.frame(
        theta = rep(c(beta, lambdas[i]), nsamples),
        parameter = rep(1:(p + 1), nsamples),
        samples = rep(1:10, each = p + 1),
        estimate = c(coef_rbmp[i, , ]),
        lambda = lambdas[i],
        method = "RBMp"
    )

    df <- rbind(df, df_m, df_mp, df_rbmp)
}

## Bias and sd in betas
summaries_beta <- df %>% filter(parameter < p + 1) %>%
    group_by(method, lambda, theta) %>%
    summarize(bias = mean(estimate - theta),
              mse = mean((estimate - theta)^2),
              var = mse - bias^2,
              mse_var = mse / var) %>%
    arrange(theta, lambda)

vars_beta <-  summaries_beta %>%
    pivot_wider(id_cols = c(lambda),
                names_from = c(method, theta),
                values_from = var) %>%
    data.frame()

mse_var_beta <- summaries_beta %>%
    pivot_wider(id_cols = lambda,
                names_from = c(method, theta),
                values_from = mse_var) %>%
    data.frame()

row_inds <- c(rbind(1:5, 5 + 1:5))
col_inds <- c(1, 1 + c(1, 4, 7, 2, 5, 8, 3, 6, 9))
result_beta <- rbind(vars_beta, mse_var_beta)[row_inds, col_inds]
memisc:::toLatex.data.frame(result_beta, digits = 2, useDcolumn = TRUE)

